# Generated by Django 2.2.2 on 2019-06-20 12:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('django_htcpcp_tea', '0002_auto_20190619_1041'),
    ]

    operations = [
        migrations.AlterField(
            model_name='teatype',
            name='name',
            field=models.CharField(db_index=True, max_length=35, unique=True, verbose_name='Tea name'),
        ),
    ]
