#  Copyright (c) 2019 Brian Schubert
#
#  This file is distributed under the MIT License. If a copy of the
#  MIT License was not distributed with this file, you can obtain one
#  at https://opensource.org/licenses/MIT.

__version__ = "0.8.1"

default_app_config = "django_htcpcp_tea.apps.HTCPCPTeaConfig"
